// Example of simple servlet allowing to browse you file system from internet broswer
import java.io.*;
import java.util.Date;
import javax.servlet.*;
import javax.servlet.http.*;

public class SimpleServlet extends HttpServlet {
    public void init() {
        // ... some initialization
    }
    
    public void destroy() {
        // ... release resources
    }
  
    protected void service(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        String methodName = req.getMethod();

        resp.addHeader("Connection", "Keep-Alive");
        resp.addHeader("Cache-Control", "no-cache");

        if (methodName.equals("GET")) { 
            String requestUri = req.getRequestURI();
            String prefix = req.getContextPath();
            if (prefix != null) {
                requestUri = requestUri.substring(prefix.length());
            }
            File file = new File(requestUri);
            if (file.exists()) {                 
                resp.setStatus(HttpServletResponse.SC_OK);
                if (file.isDirectory()) { 
                    listDirectory(file, req, resp);
                } else { 
                    dumpFile(file, req, resp);
                }
            } else { 
                resp.setStatus(HttpServletResponse.SC_NOT_FOUND);
            }
        } else { 
            resp.setStatus(HttpServletResponse.SC_METHOD_NOT_ALLOWED);
        }
         resp.flushBuffer();
   }

    static class ResponsePrintWriter extends PrintWriter {
	boolean      trace;
	StringBuffer buf;
	HttpServletResponse resp;
	
	ResponsePrintWriter(HttpServletResponse resp, boolean trace) throws IOException {
	    super(resp.getWriter());
	    buf = new StringBuffer();
	    this.trace = trace;
	    this.resp = resp;
	}
	
	public void print(String s) {
	    buf.append(s);
	}
	
	public void println(String s) {
	    buf.append(s);
	    buf.append('\n');
	}
	
	public void close() {
	    String s = buf.toString();
	    resp.setContentLength(s.length());
	    super.write(s);
	    if (trace) { 
		System.out.print(s);
	    }
	    super.close();
	}
    }

    private String getResourceHref(HttpServletRequest req, File f) { 
	String url = "http://" + req.getServerName();
	int port = req.getServerPort();
	if (port != 80) { 
	    url += ":" + port;
	}
	return url + req.getContextPath() + f.getPath().replace(File.separatorChar, '/');
    }
        

    private void listDirectory(File dir, HttpServletRequest req, HttpServletResponse resp) throws IOException {
        PrintWriter writer = new ResponsePrintWriter(resp, false);

        writer.print("<HTML><HEAD><TITLE>" + dir.getCanonicalPath() + "</TITLE><BODY><TABLE BORDER WIDTH=100%>\n");
        writer.print("<TR><TH ALIGN=LEFT>Name</TH><TH ALIGN=LEFT>Last modified</TH><TH ALIGN=LEFT>Size</TH></TR>");
        resp.setContentType("text/html; charset=\"utf-8\"");
        File[] files = dir.listFiles();
        File parent = dir.getParentFile();
        if (parent != null) { 
            writer.print("<TR><TD><A HREF=\"");
            writer.print(getResourceHref(req, parent));
            writer.print("\"><b>..</b></A></td><td>");
            writer.print(new Date(parent.lastModified()).toString());
            writer.print("</TD><TD>");
            writer.print(Long.toString(parent.length()));
            writer.print("</TD></TR>\n");
        }
        for (int i = 0; i < files.length; i++) { 
            File f = files[i];
            writer.print("<TR><TD><A HREF=\"");
            writer.print(getResourceHref(req, f));
            writer.print("\">");
            if (f.isDirectory()) { 
                writer.print("<b>");
            }
            writer.print(f.getName());
            if (f.isDirectory()) { 
                writer.print("</b>");
            }
            writer.print("</A></TD><TD>");
            writer.print(new Date(f.lastModified()).toString());
            writer.print("</TD><TD>");
            writer.print(Long.toString(f.length()));
            writer.print("</TD></TR>\n");
        }
        writer.print("</TABLE></BODY></HTML>");
        writer.close();
    }

            
    private void dumpFile(File f, HttpServletRequest req, HttpServletResponse resp) throws IOException {
        resp.setContentType("text/plain; charset=\"utf-8\"");
        OutputStream out = resp.getOutputStream();
        InputStream in = new FileInputStream(f);
        byte[] buf = new byte[4096];
        int rc;
        resp.setContentLength(in.available());
        while ((rc = in.read(buf)) > 0) { 
            out.write(buf, 0, rc);
        }
        out.close();
    }
}